package ru.ulstu.controller;

import io.swagger.v3.oas.annotations.Operation;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import ru.ulstu.configuration.ApiConfiguration;
import ru.ulstu.datamodel.exception.ModelingException;
import ru.ulstu.datamodel.ts.TimeSeries;
import ru.ulstu.db.DbService;
import ru.ulstu.db.model.TimeSeriesSet;
import ru.ulstu.estimate.CompressionMetricService;
import ru.ulstu.service.UtilService;
import ru.ulstu.target.AnomalyDifferenceSmoothed;
import ru.ulstu.target.Target;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.concurrent.ExecutionException;

@RestController
@RequestMapping(ApiConfiguration.API_1_0)
public class AnomalyController {

    private final DbService dbService;
    private final Target target;

    public AnomalyController(CompressionMetricService compressionMetricService, DbService dbService, AnomalyDifferenceSmoothed anomalyDifferenceSmoothed) {
        this.dbService = dbService;
        this.target = anomalyDifferenceSmoothed;
    }

    @GetMapping ("getAnomalyAtDifferenceSmothed")
    @Operation(description = "Получить аномальные значения")
    public TimeSeries getAnomalyAtDifferenceSmothed(@RequestParam("setKey") String setKey, @RequestParam("timeSeriesKey") String timeSeriesKey) throws ModelingException, ExecutionException, InterruptedException, InvocationTargetException, NoSuchMethodException, InstantiationException, IllegalAccessException, IOException {
        var timeSeries = dbService.getTimeSeries(new TimeSeriesSet(setKey), timeSeriesKey);
        var timeSeriesResult = target.calculate(timeSeries);
        return timeSeriesResult;
    }
}
