package ru.ulstu.controller;

import io.swagger.v3.oas.annotations.Operation;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import ru.ulstu.configuration.ApiConfiguration;
import ru.ulstu.datamodel.exception.ModelingException;
import ru.ulstu.datamodel.ts.TimeSeries;
import ru.ulstu.db.DbService;
import ru.ulstu.db.model.TimeSeriesSet;
import ru.ulstu.estimate.CompressionMetricService;
import ru.ulstu.service.PrometheusService;
import ru.ulstu.service.UtilService;
import ru.ulstu.target.AnomalyDifferenceSmoothed;
import ru.ulstu.target.Target;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.concurrent.ExecutionException;

@RestController
@RequestMapping(ApiConfiguration.API_1_0)
public class CompressionMetricController {

    private final UtilService utilService;
    private final DbService dbService;
    private final PrometheusService prometheusService;
    private final CompressionMetricService compressionMetricService;
    private final Target target;

    public CompressionMetricController(UtilService utilService, DbService dbService, PrometheusService prometheusService, CompressionMetricService compressionMetricService, AnomalyDifferenceSmoothed anomalyDifferenceSmoothed) {
        this.utilService = utilService;
        this.dbService = dbService;
        this.prometheusService = prometheusService;
        this.compressionMetricService = compressionMetricService;
        this.target = anomalyDifferenceSmoothed;
    }

    @GetMapping("getMetricAtAnomaly")
    @Operation(description = "Получить метрику сжатия")
    public ResponseEntity<TimeSeries> getMetricAtAnomaly(@RequestParam("setKey") String setKey, @RequestParam("timeSeriesKey") String timeSeriesKey) throws ModelingException, ExecutionException, InterruptedException, InvocationTargetException, NoSuchMethodException, InstantiationException, IllegalAccessException, IOException {
        var timeSeries = dbService.getTimeSeries(new TimeSeriesSet(setKey), timeSeriesKey);
        var timeSeriesResult = compressionMetricService.getCompressionTimeSeries(timeSeries);
        return new ResponseEntity<>(timeSeriesResult, HttpStatus.OK);
    }

    @GetMapping("getMetricAtAnomalyOfRandom")
    @Operation(description = "Получить метрику сжатия")
    public ResponseEntity<TimeSeries> getMetricAtAnomalyOfRandom(@RequestParam("lenght") int len) throws ModelingException, ExecutionException, InterruptedException, InvocationTargetException, NoSuchMethodException, InstantiationException, IllegalAccessException {
        var timeSeries = utilService.getRandomTimeSeries(len);
        var timeSeriesResult = compressionMetricService.getCompressionTimeSeries(timeSeries);
        return new ResponseEntity<>(timeSeriesResult, HttpStatus.OK);
    }

    @GetMapping("getMetricAtAnomalyOfPrometheus")
    @Operation(description = "Получить метрику сжатия")
    public ResponseEntity<TimeSeries> getMetricAtAnomalyOfPrometheus(@RequestParam("query") String query) throws ModelingException, ExecutionException, InterruptedException, InvocationTargetException, NoSuchMethodException, InstantiationException, IllegalAccessException {
        var timeSeries = prometheusService.executionQueryRange(query);
        var timeSeriesResult = compressionMetricService.getCompressionTimeSeries(timeSeries);
        return new ResponseEntity<>(timeSeriesResult, HttpStatus.OK);
    }
}
