package ru.ulstu.estimate;


import org.springframework.stereotype.Service;
import ru.ulstu.datamodel.Model;
import ru.ulstu.datamodel.ModelingResult;
import ru.ulstu.datamodel.exception.ModelingException;
import ru.ulstu.datamodel.ts.TimeSeries;
import ru.ulstu.datamodel.ts.TimeSeriesValue;
import ru.ulstu.method.MethodParamValue;
import ru.ulstu.method.ftransform.FTransform;
import ru.ulstu.score.ScoreMethod;
import ru.ulstu.service.MethodParamBruteForce;
import ru.ulstu.service.TimeSeriesService;
import ru.ulstu.service.UtilService;
import ru.ulstu.target.AnomalyDifferenceSmoothed;

import java.lang.reflect.InvocationTargetException;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.*;
import java.util.stream.Collectors;

@Service
public class CompressionMetricService {

    private final int DEFAULT_THREAD_COUNT = 10;
    private final TimeSeriesService timeSeriesService;
    private final FTransform fTransform;
    private final MethodParamBruteForce methodParamBruteForce;
    private final UtilService utilService;
    private final ScoreMethod scoreMethod;

    private final AnomalyDifferenceSmoothed anomalyDifferenceSmoothed;
    private final ExecutorService executors = Executors.newFixedThreadPool(DEFAULT_THREAD_COUNT);

    public CompressionMetricService(TimeSeriesService timeSeriesService, FTransform fTransform, MethodParamBruteForce methodParamBruteForce, UtilService utilService, ScoreMethod scoreMethod, AnomalyDifferenceSmoothed anomalyDifferenceSmoothed) {
        this.timeSeriesService = timeSeriesService;
        this.fTransform = fTransform;
        this.methodParamBruteForce = methodParamBruteForce;
        this.utilService = utilService;
        this.scoreMethod = scoreMethod;
        this.anomalyDifferenceSmoothed = anomalyDifferenceSmoothed;
    }

    public TimeSeries getCompressionTimeSeries(TimeSeries ts) throws ModelingException, ExecutionException, InterruptedException, InvocationTargetException, NoSuchMethodException, InstantiationException, IllegalAccessException {
        //ModelingResult modelingResult = timeSeriesService.smoothTimeSeries(ts, "FTransform");
        ModelingResult modelingResult = getCompressionTimeSeriesWithFTransform(ts);
        //var ts1 = fTransform.getForecast(ts, )
        var timeSeriesAnomalyResult = anomalyDifferenceSmoothed.calculate(modelingResult.getTimeSeries());
        var timeSeriesAnomaly = anomalyDifferenceSmoothed.calculate(ts);
        System.out.println(ts.getLength()+"\t"
                +timeSeriesAnomaly.getLength()+"\t"
                +Math.abs(timeSeriesAnomaly.getLength() - timeSeriesAnomalyResult.getLength())+"\t"
                +(double)modelingResult.getTimeSeries().getLength()/ts.getLength()+"\t"
                +modelingResult.getScore().getDoubleValue());
        System.out.println(timeSeriesAnomaly);
        System.out.println(timeSeriesAnomalyResult);
        System.out.println(utilService.getTimeSeriesToString(ts));
        System.out.println(utilService.getTimeSeriesToString(modelingResult.getTimeSeries()));
        return modelingResult.getTimeSeries();
    }

    protected ModelingResult getCompressionTimeSeriesWithFTransform(TimeSeries timeSeries) throws ExecutionException, InterruptedException {
        List<Future<ModelingResult>> results = new ArrayList<>();
        List<ModelingResult> results2 = new CopyOnWriteArrayList<>();

        Map<LocalDateTime, Double> tsValues = timeSeries.getValues().stream()
                .collect(Collectors.toMap(TimeSeriesValue::getDate, TimeSeriesValue::getValue,
                        (oldValue, newValue) -> oldValue, LinkedHashMap::new));


        List<List<MethodParamValue>> availableParametersValues = methodParamBruteForce.getAvailableParametersValues(timeSeries, fTransform.getAvailableParameters());
        for (List<MethodParamValue> parametersValues : availableParametersValues) {
            results.add(executors.submit(() -> {
                Model model = fTransform.getModel(timeSeries, parametersValues);
                return new ModelingResult(model.getTimeSeriesModel(),
                        null,
                        parametersValues,
                        scoreMethod.getScore(tsValues, model.getTimeSeriesModel()), fTransform);
            }));
        }


        for (Future<ModelingResult> futureModelingResult : results) {
            results2.add(futureModelingResult.get());
        }
        return results2.stream()
                .min(Comparator.comparing(modelingResult -> modelingResult.getScore().getDoubleValue()))
                .orElse(null);
    }

}
