package ru.ulstu.service;

import org.json.JSONObject;
import org.springframework.stereotype.Service;
import org.springframework.web.util.UriComponentsBuilder;
import ru.ulstu.datamodel.ts.TimeSeries;
import ru.ulstu.datamodel.ts.TimeSeriesValue;
import ru.ulstu.http.WebClientService;

import java.math.BigDecimal;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;

@Service
public class PrometheusService {

    private static final String PROMETHEUS_API_URL = "http://prometheus.athene.tech/api/v1/";
    private final WebClientService httpService;

    public PrometheusService(WebClientService httpService) {
        this.httpService = httpService;
    }

    public TimeSeries executionQuery(String query){ // example: pve_cpu_usage_ratio{id="lxc/111"}[5h]
        var timeSeries = new TimeSeries();
        UriComponentsBuilder builder = UriComponentsBuilder.fromHttpUrl(PROMETHEUS_API_URL + "query")
                .queryParam("query",
                        "pve_cpu_usage_ratio{id='lxc/111'}[1h]");

        var response = httpService.get(PROMETHEUS_API_URL + "query?query=" + query);
        if (response.getJSONObject(0).getString("status").equals("success")){

        }
        return timeSeries;
    }

    public TimeSeries executionQueryPost(String query){ // example: pve_cpu_usage_ratio{id="lxc/111"}[5h]
        var timeSeries = new TimeSeries();
        var postData = new JSONObject();
        postData.put("query", query);
        var response = httpService.post(PROMETHEUS_API_URL + "query", postData);
//        if (response.getJSONObject(0).getString("status").equals("success")){
//
//        }
        return timeSeries;
    }

    public TimeSeries executionQueryRange(String query){
        int numberTimeSeries = 14;
        var nowDateTime = LocalDateTime.now().truncatedTo(ChronoUnit.SECONDS);
        var timeSeries = new TimeSeries();
        var response = httpService.get(PROMETHEUS_API_URL + "query_range?query=" + query +
                "&start=" + nowDateTime.minusHours(10).minusMinutes(100).toString()+".004Z" +
                "&end=" + nowDateTime.minusHours(10).toString()+".004Z" +
                "&step=15s");
        var array = response.getJSONObject(0).getJSONObject("data").getJSONArray("result").getJSONObject(numberTimeSeries).getJSONArray("values").toList();
        for (int i = 0; i < array.size(); i++) {
            BigDecimal seconds = (BigDecimal)(((ArrayList) array.get(i)).get(0));
            Double value = Double.parseDouble((String)(((ArrayList) array.get(i)).get(1)));
            LocalDateTime dateTime =
                    LocalDateTime.ofInstant(Instant.ofEpochSecond(seconds.longValue()), ZoneId.systemDefault());
            timeSeries.addValue(new TimeSeriesValue(dateTime, value));
        }
        return timeSeries;
    }

    public TimeSeries executionQueryRangePost(String query){
        var timeSeries = new TimeSeries();
        var postData = new JSONObject();
        postData.put("query", query);
        postData.put("start", "2023-04-06T00:00:00.004Z");
        postData.put("end", "2023-04-07T00:00:00.004Z");
        postData.put("step", "15s");

        var response = httpService.post(PROMETHEUS_API_URL + "query_range", postData);
//        response.getJSONObject();
//        if (response.getJSONObject(0).getString("status").equals("success")){
//            var array = response.getJSONObject(0).getJSONObject("data").getJSONArray("result");
//            int a = 5;
//        }
        return timeSeries;
    }

}
