package ru.ulstu.target;

import org.springframework.stereotype.Service;
import ru.ulstu.datamodel.ModelingResult;
import ru.ulstu.datamodel.exception.ModelingException;
import ru.ulstu.datamodel.ts.TimeSeries;
import ru.ulstu.datamodel.ts.TimeSeriesValue;
import ru.ulstu.service.TimeSeriesService;

import java.lang.reflect.InvocationTargetException;
import java.util.concurrent.ExecutionException;

@Service
public class AnomalyDifferenceSmoothed extends Target {

    private final TimeSeriesService timeSeriesService;

    public AnomalyDifferenceSmoothed(TimeSeriesService timeSeriesService) {
        super("Модель сравнения со сглаженным ВР");
        this.timeSeriesService = timeSeriesService;
    }

    @Override
    public TimeSeries calculate(TimeSeries timeSeries) throws ModelingException, ExecutionException, InterruptedException, InvocationTargetException, NoSuchMethodException, InstantiationException, IllegalAccessException {
        TimeSeries residual = new TimeSeries();
        TimeSeries anomalyPoints = new TimeSeries();

        TimeSeries timeSeriesSmooth = timeSeriesService.smoothTimeSeries(timeSeries, "AddTrendAddSeason").getTimeSeries();

        for (int i = 0; i < timeSeries.getLength(); i++) {
            residual.addValue(new TimeSeriesValue(timeSeries.getValue(i).getDate(),
                    Math.abs(timeSeries.getNumericValue(i) - timeSeriesSmooth.getNumericValue(i))));
        }

        double diff = timeSeries.getMax().getValue()-timeSeries.getMin().getValue();

        for (int i = 0; i < residual.getLength(); i++) {
            if (residual.getNumericValue(i)/diff > 0.05) {
                anomalyPoints.addValue(new TimeSeriesValue(residual.getValue(i).getDate(),
                        timeSeries.getNumericValue(i)));
            }
        }

        return anomalyPoints;
    }
}